@@ -2,6 +2,7 @@ package Moo::Role;
 
 use strictures 1;
 use Moo::_Utils;
+use Role::Tiny ();
 use base qw(Role::Tiny);
 
 require Moo::sification;
@@ -9,6 +10,7 @@ require Moo::sification;
 BEGIN { *INFO = \%Role::Tiny::INFO }
 
 our %INFO;
+our %APPLY_DEFAULTS;
 
 sub _install_tracked {
   my ($target, $name, $code) = @_;
@@ -99,10 +101,12 @@ sub _inhale_if_moose {
       and (
         $INC{"Moose.pm"}
         and $meta = Class::MOP::class_of($role)
+        and $meta->isa('Moose::Meta::Role')
       )
       or (
         Mouse::Util->can('find_meta')
         and $meta = Mouse::Util::find_meta($role)
+        and $meta->isa('Mouse::Meta::Role')
      )
   ) {
     $INFO{$role}{methods} = {
@@ -116,12 +120,14 @@ sub _inhale_if_moose {
     $INFO{$role}{requires} = [ $meta->get_required_method_list ];
     $INFO{$role}{attributes} = [
       map +($_ => do {
-        my $spec = { %{$meta->get_attribute($_)} };
+        my $attr = $meta->get_attribute($_);
+        my $is_mouse = $meta->isa('Mouse::Meta::Role');
+        my $spec = { %{ $is_mouse ? $attr : $attr->original_options } };
 
         if ($spec->{isa}) {
 
           my $get_constraint = do {
-            my $pkg = $meta->isa('Mouse::Meta::Role')
+            my $pkg = $is_mouse
                         ? 'Mouse::Util::TypeConstraints'
                         : 'Moose::Util::TypeConstraints';
             _load_module($pkg);
@@ -162,29 +168,30 @@ sub _inhale_if_moose {
     }
     require Class::Method::Modifiers if @$mods;
     $INFO{$role}{inhaled_from_moose} = 1;
+    $INFO{$role}{is_role} = 1;
   }
 }
 
 sub _maybe_make_accessors {
-  my ($self, $role, $target) = @_;
+  my ($self, $target, $role) = @_;
   my $m;
-  if ($INFO{$role}{inhaled_from_moose}
+  if ($INFO{$role} && $INFO{$role}{inhaled_from_moose}
       or $INC{"Moo.pm"}
       and $m = Moo->_accessor_maker_for($target)
       and ref($m) ne 'Method::Generate::Accessor') {
-    $self->_make_accessors($role, $target);
+    $self->_make_accessors($target, $role);
   }
 }
 
 sub _make_accessors_if_moose {
-  my ($self, $role, $target) = @_;
-  if ($INFO{$role}{inhaled_from_moose}) {
-    $self->_make_accessors($role, $target);
+  my ($self, $target, $role) = @_;
+  if ($INFO{$role} && $INFO{$role}{inhaled_from_moose}) {
+    $self->_make_accessors($target, $role);
   }
 }
 
 sub _make_accessors {
-  my ($self, $role, $target) = @_;
+  my ($self, $target, $role) = @_;
   my $acc_gen = ($Moo::MAKERS{$target}{accessor} ||= do {
     require Method::Generate::Accessor;
     Method::Generate::Accessor->new
@@ -200,10 +207,16 @@ sub _make_accessors {
   }
 }
 
+sub role_application_steps {
+  qw(_handle_constructor _maybe_make_accessors),
+    $_[0]->SUPER::role_application_steps;
+}
+
 sub apply_roles_to_package {
   my ($me, $to, @roles) = @_;
   foreach my $role (@roles) {
-      $me->_inhale_if_moose($role);
+    $me->_inhale_if_moose($role);
+    die "${role} is not a Moo::Role" unless $INFO{$role};
   }
   $me->SUPER::apply_roles_to_package($to, @roles);
 }
@@ -211,8 +224,7 @@ sub apply_roles_to_package {
 sub apply_single_role_to_package {
   my ($me, $to, $role) = @_;
   $me->_inhale_if_moose($role);
-  $me->_handle_constructor($to, $INFO{$role}{attributes});
-  $me->_maybe_make_accessors($role, $to);
+  die "${role} is not a Moo::Role" unless $INFO{$role};
   $me->SUPER::apply_single_role_to_package($to, $role);
 }
 
@@ -244,23 +256,61 @@ sub create_class_with_roles {
   $me->SUPER::create_class_with_roles($superclass, @roles);
 
   foreach my $role (@roles) {
-    die "${role} is not a Role::Tiny" unless my $info = $INFO{$role};
+    die "${role} is not a Role::Tiny" unless $INFO{$role};
   }
 
-  $Moo::MAKERS{$new_name} = {};
+  $Moo::MAKERS{$new_name} = {is_class => 1};
 
-  $me->_handle_constructor(
-    $new_name, [ map @{$INFO{$_}{attributes}||[]}, @roles ], $superclass
-  );
+  $me->_handle_constructor($new_name, $_) for @roles;
 
   return $new_name;
 }
 
+sub apply_roles_to_object {
+  my ($me, $object, @roles) = @_;
+  my $new = $me->SUPER::apply_roles_to_object($object, @roles);
+
+  my $apply_defaults = $APPLY_DEFAULTS{ref $new} ||= do {
+    my %attrs = map { @{$INFO{$_}{attributes}||[]} } @roles;
+
+    if ($INC{'Moo.pm'}
+        and keys %attrs
+        and my $con_gen = Moo->_constructor_maker_for(ref $new)
+        and my $m = Moo->_accessor_maker_for(ref $new)) {
+      require Sub::Quote;
+
+      my $specs = $con_gen->all_attribute_specs;
+
+      my $assign = '';
+      my %captures;
+      foreach my $name ( keys %attrs ) {
+        my $spec = $specs->{$name};
+        if ($m->has_eager_default($name, $spec)) {
+          my ($has, $has_cap)
+            = $m->generate_simple_has('$_[0]', $name, $spec);
+          my ($code, $pop_cap)
+            = $m->generate_use_default('$_[0]', $name, $spec, $has);
+
+          $assign .= $code;
+          @captures{keys %$has_cap, keys %$pop_cap}
+            = (values %$has_cap, values %$pop_cap);
+        }
+      }
+      Sub::Quote::quote_sub($assign, \%captures);
+    }
+    else {
+      sub {};
+    }
+  };
+  $new->$apply_defaults;
+  return $new;
+}
+
 sub _composable_package_for {
   my ($self, $role) = @_;
   my $composed_name = 'Role::Tiny::_COMPOSABLE::'.$role;
   return $composed_name if $Role::Tiny::COMPOSED{role}{$composed_name};
-  $self->_make_accessors_if_moose($role, $composed_name);
+  $self->_make_accessors_if_moose($composed_name, $role);
   $self->SUPER::_composable_package_for($role);
 }
 
@@ -270,14 +320,15 @@ sub _install_single_modifier {
 }
 
 sub _handle_constructor {
-  my ($me, $to, $attr_info, $superclass) = @_;
+  my ($me, $to, $role) = @_;
+  my $attr_info = $INFO{$role} && $INFO{$role}{attributes};
   return unless $attr_info && @$attr_info;
   if ($INFO{$to}) {
     push @{$INFO{$to}{attributes}||=[]}, @$attr_info;
   } else {
     # only fiddle with the constructor if the target is a Moo class
     if ($INC{"Moo.pm"}
-        and my $con = Moo->_constructor_maker_for($to, $superclass)) {
+        and my $con = Moo->_constructor_maker_for($to)) {
       # shallow copy of the specs since the constructor will assign an index
       $con->register_attribute_specs(map ref() ? { %$_ } : $_, @$attr_info);
     }
@@ -341,7 +392,7 @@ L<Moo/has> for all options.
 
 =head1 SUPPORT
 
-See L<Moo> for support and contact informations.
+See L<Moo> for support and contact information.
 
 =head1 AUTHORS
 
